@@ -1,6 +1,6 @@
 package Asyncore;
 {
-  $Asyncore::VERSION = '0.06';
+  $Asyncore::VERSION = '0.08';
 }
 
 #==============================================================================
@@ -14,7 +14,7 @@ package Asyncore;
 #        NOTES:  ---
 #       AUTHOR:   (Sebastiano Piccoli), <sebastiano.piccoli@gmail.com>
 #      COMPANY:  
-#      VERSION:  0.06
+#      VERSION:  0.08
 #      CREATED:  26/06/12 20:27:28 CEST
 #     REVISION:  ---
 #==============================================================================
@@ -23,6 +23,8 @@ use strict;
 use warnings;
 
 use IO::Select;
+use Socket;
+use Carp;
 
 our $socket_map;
 
@@ -101,11 +103,12 @@ sub poll {
             ($rr, $rw, $he) = IO::Select->select($r, $w, $e, $timeout);
         };
         if ($@) {
-            die "error in select(r, w, e): " . $@;
+            croak "$@";
         }
         
         foreach my $fd (@$rr) {
-            my $obj = $map->{$fd->fileno()};
+            next if !defined(fileno($fd));
+            my $obj = $map->{fileno($fd)};
             if (not $obj) {
                 next;
             }
@@ -113,7 +116,8 @@ sub poll {
         }
         
         foreach my $fd (@$rw) {
-            my $obj = $map->{$fd->fileno()};
+            next if !defined(fileno($fd));
+            my $obj = $map->{fileno($fd)};
             if (not $obj) {
                 next;
             }
@@ -166,7 +170,7 @@ sub loop {
 
 package Asyncore::Dispatcher;
 {
-    $Asyncore::Dispatcher::VERSION = '0.06';
+    $Asyncore::Dispatcher::VERSION = '0.08';
 }
 
 #==============================================================================
@@ -181,7 +185,7 @@ package Asyncore::Dispatcher;
 #        NOTES:  ---
 #       AUTHOR:   (Sebastiano Piccoli), <sebastiano.piccoli@gmail.com>
 #      COMPANY:  
-#      VERSION:  0.6
+#      VERSION:  0.8
 #      CREATED:  26/06/12 20:27:28 CEST
 #     REVISION:  ---
 #==============================================================================
@@ -189,7 +193,9 @@ package Asyncore::Dispatcher;
 use strict;
 use warnings;
 
-use IO::Socket::INET6;
+use Socket;
+use Fcntl;
+use Carp;
 use Errno;
 
 sub new {
@@ -210,15 +216,15 @@ sub init {
     }
 
     $self->{_fileno} = 0;
-
+    
     if ($sock) {
         # Set to nonblocking just to make sure for cases where we 
         # get a socket from a blocking source
-        $sock->blocking(0);
+        fcntl($sock, F_SETFL, O_NONBLOCK);
         $self->set_socket($sock, $map);
         $self->{_connected} = 1;
         
-        if (not $sock->peername()) {
+        if (not getpeername($sock)) {
             if ($!{ENOTCONN} || $!{EINVAL}) {
                 # Handle the case where we got an unconnected socket
                 $self->{_connected} = 0;
@@ -228,6 +234,7 @@ sub init {
                 # way, alert the user and remove it from the map (to prevent
                 # polling of broken sockets)
                 $self->del_channel($map);
+        
             }
         }
     }
@@ -266,7 +273,7 @@ sub del_channel {
 }   
 
 sub create_socket {
-    my($self, $port, $family, $type) = @_;
+    my($self, $family, $type) = @_;
     
     if (not $family) {
         $family = AF_INET;
@@ -276,20 +283,19 @@ sub create_socket {
     }
     # SOCK_DGRAM (udp), SOCK_RAW (icmp)
     
-    my $sock = IO::Socket::INET6->new(LocalAddr => '',
-                                      LocalPort => $port,
-                                      Domain => $family,
-                                      Type => $type,
-                                      Blocking => 0);
+    my $sock;
+    my $proto = getprotobyname("tcp"); # fixme
+    socket($sock, AF_INET, SOCK_STREAM, $proto) || croak "$!";
+    fcntl($sock, F_SETFL, O_NONBLOCK);
+    
     $self->set_socket($sock);
-
 }
 
 sub set_socket {
     my($self, $sock, $map) = @_;
 
     $self->{_socket} = $sock;
-    $self->{_fileno} = $sock->fileno();
+    $self->{_fileno} = fileno($sock);
     $self->add_channel($map);
 }
     
@@ -305,27 +311,59 @@ sub writable {
     return 1;
 }
 
- 
+sub bind {
+    my($self, $port) = @_;
+    
+    my $sock = $self->{_socket};
+    my $iaddr = gethostbyname('localhost');
+    bind($sock, sockaddr_in($port, $iaddr));
+}
+
 sub listen {
     my($self, $num) = @_;
 
     $self->{_accepting} = 1;
-    # if os == nt then n max = 5
-    return $self->{_socket}->listen($num); 
+
+    my $sock = $self->{_socket};
+    listen($sock, $num);
+}
+
+sub connect {
+    my($self, $addr, $port) = @_;
+
+    $self->{_connected} = 0;
+    $self->{_connecting} = 1;
+
+    my $sock = $self->{_socket};
+    my $iaddr = inet_aton($addr);
+    my $paddr = sockaddr_in($port, $iaddr);
+    # In the case of a non-blocking socket connection cannot be completed
+    # immediately. connect is a form of write so select() or poll() can be
+    # used to determine when it's possible to write.
+    unless (connect($sock, $paddr)) {
+        if ($!{EINPROGRESS}) {
+            return;
+        }
+    }
+    
+    $self->{_addr} = $addr;
+    $self->{_port} = $port;
+    $self->handle_connect_event();
 }
 
 sub accept {
     my $self = shift;
     
     my $conn;
+    my $sock = $self->{_socket};
     
-    if (not $conn = $self->{_socket}->accept()) {
+    if (not accept($conn, $sock)) {
         if ($!{EWOULDBLOCK} || $!{ECONNABORTED} || $!{EAGAIN}) {
             # Handle the case where we cannot accept connection
-            return
+            return;
         }
         else {
-            warn 'unknown error in accept()';
+            carp 'Unknown error in accept()';
         }        
     }
     
@@ -335,30 +373,35 @@ sub accept {
 sub send {
     my($self, $data) = @_;
     
+    my $sock = $self->{_socket};
+    
     my $result;
     eval {
-        $result = $self->{_socket}->send($data);
+        send($sock, $data, 0);
     };
-    #if ($@) {
-    #    
-    #}
+    if ($@) {
+        croak "$@";
+    }
 
     return $result;
 }
 
-sub recv {
+sub receive {
     my($self, $buffer_size) = @_;
     
-    my $data = $self->{_socket}->recv('', $buffer_size);
+    my $data;
+    my $sock = $self->{_socket};
+    recv($sock, $data, $buffer_size, 0);
+    
     if (not $data) {
         $self->handle_close();
-        return ''
+        return '';
     }
     else {
         return $data;
     }
     
-    # check error
+    # check error todo
 }
 
 sub close {
@@ -367,7 +410,9 @@ sub close {
     $self->{_connected} = 0;
     $self->{_accepting} = 0;
     $self->{_connecting} = 0;
-    $self->{_socket}->close();
+    
+    my $sock = $self->{_socket};
+    close($sock);
 }
 
 sub handle_close {
@@ -396,7 +441,7 @@ sub handle_read_event {
 sub handle_connect_event {
     my $self = shift;
     
-    # some preliminary check
+    # some preliminary check todo
     
     $self->handle_connect();
     $self->{_connected} = 1;
@@ -407,7 +452,7 @@ sub handle_write_event {
     my $self = shift;
     
     if ($self->{_accepting}) {
-        return
+        return;
     }
     
     if (not $self->{_connected}) {
@@ -474,7 +519,8 @@ Asyncore - basic infrastracture for asynchronous socket services
     use base qw( Asyncore::Dispatcher );
     
     my $server = Asyncore::Dispatcher->new();
-    $server->create_socket(35000);
+    $server->create_socket();
+    $server->bind($port)
     $server->listen(5);
  
     Asyncore::loop();
@@ -505,7 +551,7 @@ To manage an asyncronous socket handler instantiate a subclass of Asyncore::Disp
  
 Enter a polling loop that terminates after count passes or all open channels have been closed. All arguments are optional. The count parameter defaults to undef, resulting in the loop terminating only when all channels have been closed. The timeout argument sets the timeout parameter for the appropriate select() or poll() call, measured in seconds; the default is 30 seconds. [The use_poll parameter, if true, indicates that poll() should be used in preference to select() (the default is False) TBD].
  
-The map parameter is an hash reference whose items are the channels to watch. As channels are closed they are deleted from their map. If map is omitted, a global map is used. [Channels (instances of Asyncore::Dispatcher, Asynchat::async_chat() and subclasses thereof) can freely be mixed in the map. TBD]
+The map parameter is an hash reference whose items are the channels to watch. As channels are closed they are deleted from their map. If map is omitted, a global map is used. Channels (instances of Asyncore::Dispatcher, Asynchat::async_chat() and subclasses thereof) can freely be mixed in the map.
 
 =head2 handle_connect()
  
@@ -519,4 +565,4 @@ This module is a porting of asyncore.py written in python.
  
 =head1 LICENCE
 
-LGPL
\ No newline at end of file
+LGPL
